##
# Copyright (c) 2014 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


from metamagic.json import loads, loadb


class TestJsonDecode:
    def test_json_loads(self):
        assert loads('{"a":"b"}') == {'a': 'b'}

    def test_json_loadb(self):
        assert loadb(b'{"a":"b"}') == {'a': 'b'}



##
# Copyright (c) 2012 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


__all__ = ('dumps', 'dumpb', 'loads', 'loadb')


try:
    from ._encoder import Encoder
except ImportError:
    from .encoder import Encoder


import json as std_json


def dumps(obj, encoder=Encoder):
    """Return a JSON representation of ``obj`` in a Python string.

       Uses the given Encoder class which is supposed to have a ``dumps``
       method.

       By default tries to use the C version of the Encoder class from the
       ``_encoder`` module. If there is no C version uses the Python version
       from the ``encoder`` module.

       See documentation for the given Encoder class for details.

       **Examples**:

       .. code-block:: pycon

           >>> dumps(True)
           'true'

           >>> dumps('foo')
           '"foo"'

       Using custom ``__mm_serialize__`` (see the Encoder class docs for more
       info):

       .. code-block:: pycon

           >>> class Foo:
           ...     def __mm_serialize__(self):
           ...         return 'foo/bar'

           >>> dumps(Foo())
           '"foo\/bar"'

       Using ``encode_hook()`` (see the Encoder class docs for more info):

       .. code-block:: pycon

           >>> class MyEncoder(Encoder):
           ...     def encode_hook(self, obj):
           ...         if isinstance(obj, int):
           ...             return '*' + str(obj)
           ...         return obj

           >>> MyEncoder().dumps([1])
           '["*1"]'
    """
    return Encoder().dumps(obj)


def dumpb(obj, encoder=Encoder):
    """Return a JSON representation of ``obj`` in a bytes() array.

       Uses the given Encoder class which is supposed to have a ``dumpb``
       method.

       By default tries to use the C version of the Encoder class from the
       ``_encoder`` module. If there is no C version uses the Python version
       from the ``encoder`` module.

       See documentation for the given Encoder class for details.

       **Examples**:

       .. code-block:: pycon

           >>> dumpb(True)
           b'true'
    """
    return Encoder().dumpb(obj)


def loads(s):
    """Deserialize ``s`` (instance of ``str``) to a Python object."""
    return std_json.loads(s)


def loadb(b):
    """Deserialize ``b`` (instance of ``bytes``) to a Python object."""
    assert isinstance(b, (bytes, bytearray))
    return std_json.loads(b.decode('utf-8'))



##
# Copyright (c) 2012 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


import types
from re import compile as re_compile
from numbers import Number
from decimal import Decimal
from math import isnan, isinf
from collections import OrderedDict, Set, Sequence, Mapping
from uuid import UUID
from datetime import date, time


JAVASCRIPT_MAXINT = 9007199254740992  # see http://ecma262-5.com/ELS5_HTML.htm#Section_8.5

BASE_ESCAPE_ASCII = re_compile(r'([\\/]|[^\ -~]|[<>&])')
ESCAPE_ASCII = re_compile(r'([\\"/]|[^\ -~]|[<>&])')

BASE_ESCAPE_DCT = {}
for i in range(0x20):
    BASE_ESCAPE_DCT[chr(i)]= '\\u{0:04x}'.format(i)
BASE_ESCAPE_DCT.update({
    '\\': '\\\\',
    '/':  '\/',
    '\b': '\\b',
    '\f': '\\f',
    '\n': '\\n',
    '\r': '\\r',
    '\t': '\\t',
})

ESCAPE_DCT = {'"': '\\"'}
ESCAPE_DCT.update(BASE_ESCAPE_DCT)


class Encoder:
    """A Python implementation of a JSON encoder for Python objects designed
       to be compatible with native JSON decoders in various web browsers.

       Can either encode to a python string (see ``dumps``) or a sequence
       of bytes (see ``dumpb``). The string returned by dumps() is guaranteed
       to have only 7-bit ASCII characters [#f1]_ and ``dumps(obj).encode('ascii') = dumpb(obj)``.

       Supports a special encoder class method `encode_hook(obj)` which, if present, is applied to
       the input object and the rest of the processing is applied to the output of encode_hook().
       Note: encode_hook() should always return an object; for objects which should not be
       specially encoded encode_hook() should return the original object.

       Supports custom encoders by using objects' ``__mm_json__()`` or ``__mm_serialize__()``
       method, if available. It is guaranteed that for all non-native types __mm_json__ and
       then __mm_serialize__ will be tried before any other attempt to encode the object [#f2]_.
       The output of __mm_serialize__ is in turn encoded as any other object (and may in turn have
       an __mm_serialize__ method or not be supported).

       Natively supports strings, integers, floats, True, False, None, lists, tuples,
       dicts, sets, frozensets, collections.OrderedDicts, colections.Set,
       collections.Sequence [#f3]_, collections.Mapping, uuid.UUIDs [#f4]_, decimal.Decimals,
       datetime.datetime and objects derived form all listed objects.

       For all objects which could not be encoded in any other way an
       attempt is made to convert an object to an encodeable one using ``self.default(obj)``
       method (which can be overwrite in derived classes). If self.default succeeds,
       the output is again encoded as any other object.


       Exceptions raised:

       * Both ``dumps()`` and ``dumpb()`` raise a TypeError for unsupported objects and
         for all dictionary keys which are not strings (or UUIDs [#f5]_) and
         which are not representable as strings (or UUIDs) by their __mm_serialize__ method.

       * ``default()`` raises a TypeError for all unsupported objects, and overwritten ``default()``
         is also expected to raise a TypeError for all objects it does not support.

       * When encoding integers, ``dumps()`` and ``dumpb()`` raise a ValueError if integer
         value is greater than the maximum integer value supported by JavaScript
         (``9007199254740992``, see http://ecma262-5.com/ELS5_HTML.htm#Section_8.5).

       * When encoding a nested object a ValueError is raised when going deeper than
         the allowed nesting level (100 by default, can be overwritten by passing the
         desired value as the second argument to ``dumps()`` and ``dumpb()`` methods)


       .. [#f1] All characters required to be escaped by the JSON spec @ http://json.org are escaped
       .. [#f2] If present, encode_hook() is applied before and independently of all other encoders
       .. [#f3] To avoid errors in the metamagic framework ``bytes()``, ``bytearray()`` and derived
                classes are deliberately not encoded using the built-in sequence encoder;
                the only way to encode these objects is to either overwrite the encoders' default()
                method or to provide __mm_serialize__ method in the object being serialized.
       .. [#f4] UUIDs and Decimals are encoded as strings.
       .. [#f5] JSON specification only supports string dictionary keys; since UUIDs
                are also encoded to strings and are a common key in the metamagic framework,
                this encoder also supports UUIDs as dictionary keys.
    """

    _nested_level     = 0            # current recursion level
    _max_nested_level = 100          # max allowed level
    _use_hook        = False

    def __init__(self):
        # If 'Encoder.encode_hook' wasn't overridden then don't call it.
        #
        func = self.encode_hook
        if isinstance(func, types.MethodType):
            func = func.__func__
        self._use_hook = func is not Encoder.encode_hook

    def encode_hook(self, obj):
        """Override this method to hook in the encoding process.  Should either
        return a modified/coerced or the same ``obj`` argument.
        """
        return obj

    def default(self, obj):
        """In this implementation always raises a TypeError.

           In order to support new object types can be overwritten to return an
           object natively supported by the encoder (see class description for
           the list of supported objects).

           Example::

            # try to import the C version, if available
            try:
                from metamagic.json._encoder import Encoder
            except ImportError:
                # C version is unavailable - import Python version
                from metamagic.json.encoder import Encoder

            class Bar:
                pass

            class MyEncoder(Encoder):
                def default(self, obj):
                    if isinstance(obj, Bar):
                        return ['Bar']
                    return super().default(obj)

            MyEncoder().dumps([Bar()]) == '[["Bar"]]'
        """
        raise TypeError('{!r} is not JSON serializable by this encoder'.format(obj))

    def _increment_nested_level(self):
        self._nested_level += 1
        if (self._nested_level > self._max_nested_level):
            raise ValueError('Exceeded maximum allowed recursion level ({}), ' \
                             'possibly circular reference detected'.format(self._max_nested_level))

    def _decrement_nested_level(self):
        self._nested_level -= 1

    def _encode_str(self, obj, escape_quotes=True):
        """Return an ASCII-only JSON representation of a Python string"""
        def replace(match):
            s = match.group(0)
            try:
                if escape_quotes:
                    return ESCAPE_DCT[s]
                else:
                    return BASE_ESCAPE_DCT[s]
            except KeyError:
                n = ord(s)
                if n < 0x10000:
                    return '\\u{0:04x}'.format(n)
                else:
                    # surrogate pair
                    n -= 0x10000
                    s1 = 0xd800 | ((n >> 10) & 0x3ff)
                    s2 = 0xdc00 | (n & 0x3ff)
                    return '\\u{0:04x}\\u{1:04x}'.format(s1, s2)
        if escape_quotes:
            return '"' + ESCAPE_ASCII.sub(replace, obj) + '"'
        else:
            return BASE_ESCAPE_ASCII.sub(replace, obj)

    def _encode_numbers(self, obj):
        """Returns a JSON representation of a Python number (int, float or Decimal)"""

        # strict checks first - for speed
        if obj.__class__ is int:
            if abs(obj)>JAVASCRIPT_MAXINT:
                raise ValueError('Number out of range: {!r}'.format(obj))
            return str(obj)

        if obj.__class__ is float:
            if isnan(obj):
                raise ValueError('NaN is not supported')
            if isinf(obj):
                raise ValueError('Infinity is not supported')
            return repr(obj)

        # more in-depth class analysys last
        if isinstance(obj,int):
            if abs(obj)>JAVASCRIPT_MAXINT:
                raise ValueError('Number out of range: {!r}'.format(obj))
            return str(obj)

        if isinstance(obj,float):
            if isnan(obj):
                raise ValueError('NaN is not supported')
            if isinf(obj):
                raise ValueError('Infinity is not supported')
            return repr(obj)

        if isinstance(obj, Decimal):
            return '"' + str(obj) + '"'

        # for complex and other Numbers
        return self._encode(self.default(obj))

    def _encode_list(self, obj):# do
        """Returns a JSON representation of a Python list"""

        self._increment_nested_level()

        buffer = []
        for element in obj:
            buffer.append(self._encode(element))

        self._decrement_nested_level()

        return '['+ ','.join(buffer) + ']'

    def _encode_dict(self, obj):
        """Returns a JSON representation of a Python dict"""

        self._increment_nested_level()

        buffer = []
        for key in obj:
            buffer.append(self._encode_key(key) + ':' + self._encode(obj[key]))

        self._decrement_nested_level()

        return '{'+ ','.join(buffer) + '}'

    def _encode_key(self, obj):
        """Encodes a dictionary key - a key can only be a string in std JSON"""

        if obj.__class__ is str:
            return self._encode_str(obj)

        if obj.__class__ is UUID:
            return '"' + str(obj) + '"'

        # __mm_serialize__ is called before any isinstance checks (but after exact type checks)
        try:
            sx_encoder = obj.__mm_serialize__
        except AttributeError:
            pass
        else:
            try:
                data = sx_encoder()
            except NotImplementedError:
                pass
            else:
                return self._encode_key(data)

        if isinstance(obj, UUID):
            return '"' + str(obj) + '"'

        if isinstance(obj, str):
            return self._encode_str(obj)

        # if everything else failed try the default() method and re-raise any TypeError
        # exceptions as more specific "not a valid dict key" TypeErrors
        try:
            value = self.default(obj)
        except TypeError:
            raise TypeError('{!r} is not a valid dictionary key'.format(obj))

        return self._encode_key(value)

    def _encode(self, obj):
        """Returns a JSON representation of a Python object - see dumps.
        Accepts objects of any type, calls the appropriate type-specific encoder.
        """

        if self._use_hook:
            obj = self.encode_hook(obj)

        # first try simple strict checks

        _objtype = obj.__class__

        if _objtype is str:
            return self._encode_str(obj)

        if _objtype is bool:
            if obj:
                return 'true'
            else:
                return 'false'

        if _objtype is int or _objtype is float:
            return self._encode_numbers(obj)

        if _objtype is list or _objtype is tuple:
            return self._encode_list(obj)

        if obj is None:
            return 'null'

        if _objtype is dict or obj is OrderedDict:
            return self._encode_dict(obj)

        if _objtype is UUID:
            return '"' + str(obj) + '"'

        if _objtype is Decimal:
            return '"' + str(obj) + '"'

        # For all non-std types try __mm_json__ and then __mm_serialize__ before any isinstance
        # checks

        try:
            sx_json_data = obj.__mm_json__
        except AttributeError:
            pass
        else:
            try:
                data = sx_json_data()
            except NotImplementedError:
                pass
            else:
                if isinstance(data, bytes):
                    return data.decode('utf-8')
                else:
                    return self._encode_str(data, escape_quotes=False)
        try:
            sx_encoder = obj.__mm_serialize__
        except AttributeError:
            pass
        else:
            try:
                data = sx_encoder()
            except NotImplementedError:
                pass
            else:
                return self._encode(data)

        # do more in-depth class analysis

        if isinstance(obj, UUID):
            return '"' + str(obj) + '"'

        if isinstance(obj, str):
            return self._encode_str(obj)

        if isinstance(obj, (list, tuple, set, frozenset, Set)):
            return self._encode_list(obj)

        if isinstance(obj, Sequence) and not isinstance(obj, (bytes, bytearray)):
            return self._encode_list(obj)

        if isinstance(obj, (dict, OrderedDict, Mapping)):
            return self._encode_dict(obj)

        # note: number checks using isinstance should come after True/False checks
        if isinstance(obj, Number):
            return self._encode_numbers(obj)

        if isinstance(obj, (date, time)):
            return '"' + obj.isoformat() + '"'

        return self._encode(self.default(obj))

    def dumps(self, obj, *, max_nested_level=100):
        """Returns a string representing a JSON-encoding of ``obj``.

           The second optional ``max_nested_level`` argument controls the maximum
           allowed recursion/nesting level.

           See class description for details.
        """
        self._max_nested_level = max_nested_level
        return self._encode(obj)

    def dumpb(self, obj, *, max_nested_level=100):
        """Similar to ``dumps()``, but returns ``bytes`` instead of a ``string``"""
        self._max_nested_level = max_nested_level
        return self._encode(obj).encode('utf-8')



##
# Copyright (c) 2012-2013 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


from collections import OrderedDict
from decimal import Decimal
from json import dumps as std_dumps
import random
import marshal

from metamagic.json._encoder import Encoder as CEncoder
from metamagic.json.encoder import Encoder as PyEncoder
from metamagic.test import benchmark, skip


class BaseBenchmarkJSONEncoder:
    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_short_ascii(self):
        arr = []
        for _ in range(256):
            arr.append("A pretty long string which is in a list")

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_2048_3_char_ascii(self):
        arr = []
        for _ in range(2048):
            arr.append("abc")

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_long_ascii(self):
        arr = []
        for _ in range(2048):
            arr.append("abcabc" + "z" * 150)

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_long_utf8(self):
        arr = []
        for _ in range(2048):
            arr.append("عالم " * 50)

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_medium_complex_object(self):
        user        = { "userId": 3381293, "age": 213, "username": "johndoe",
                        "fullname": "John Doe the Second", "isAuthorized": True,
                        "liked": 31231.31231202, "approval": 31.1471,
                        "jobs": [ 1, 2 ], "currJob": None }
        friends     = [ user, user, user, user, user, user, user, user ]
        testobj     = [ [user, friends],  [user, friends],  [user, friends],
                        [user, friends],  [user, friends],  [user, friends] ]

        return lambda: self.encode(testobj)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_doubles(self):
        arr = []
        for _ in range(256):
            arr.append(10000000 * random.random())

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_ints(self):
        arr = []
        for _ in range(256):
            arr.append(int(10000000 * random.random()))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_small_ints(self):
        arr = []
        for _ in range(256):
            arr.append(int(10000 * random.random()))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_decimals(self):
        arr = []
        for _ in range(256):
            arr.append(Decimal(str(random.random()*100000)))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_true_false_values(self):
        arr = []
        for _ in range(128):
            arr.extend((True, False))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_dict_string_int(self):
        arr = []
        for _ in range(128):
            arr.append({str(random.random()*20): int(random.random()*1000000)})

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_deriveddict_string_int(self):
        class DerivedDict(dict):
            pass
        arr = []
        for _ in range(128):
            arr.append(DerivedDict({str(random.random()*20): int(random.random()*1000000)}))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_ordereddict_string_int(self):
        arr = []
        for _ in range(128):
            d = {
                str(random.random()*20): int(random.random()*1000000),
                str(random.random()*20): int(random.random()*1000000),
                str(random.random()*20): int(random.random()*1000000),
                str(random.random()*20): int(random.random()*1000000)
            }
            ordered_d = OrderedDict(sorted(d.items(), key=lambda t: t[0]))
            arr.append(ordered_d)

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=6.0)
    def benchmark_dict_256_arrays_256_string_int_pairs(self):
        dct = {}
        for _ in range(128):
            arrays = []
            for _ in range(256):
                arrays.append({str(random.random()*20): int(random.random()*1000000)})
            dct[str(random.random()*20)] = arrays

        return lambda: self.encode(dct)


class BaseBenchmarkJSONEncoderCustom:
    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_objs_with_mm_serialize(self):
        class CustomObject:
            def __init__(self, a, b):
                self.a = a
                self.b = b

            def __mm_serialize__(self):
                return {"a": self.a, "b": self.b}

        arr = []
        for _ in range(256):
            arr.append(CustomObject(a = str(random.random()*20), b = int(random.random()*20)))

        return lambda: self.encode(arr)

    @benchmark.throughput(seconds=3.0)
    def benchmark_array_256_objs_with_mm_json(self):
        class CustomObject:
            def __mm_json__(self):
                return b'{"a": "spamspamspam!", "b": 42424242}'

        arr = []
        for _ in range(256):
            arr.append(CustomObject())

        return lambda: self.encode(arr)


class BenchmarkJSONEncoder_Std(BaseBenchmarkJSONEncoder):
    def encode(self, obj):
        return std_dumps(obj)

    def benchmark_array_256_decimals(self):
        skip()


class BenchmarkJSONEncoder_C(BaseBenchmarkJSONEncoder, BaseBenchmarkJSONEncoderCustom):
    def encode(self, obj):
        return CEncoder().dumpb(obj)


class BenchmarkJSONEncoder_Python(BaseBenchmarkJSONEncoder, BaseBenchmarkJSONEncoderCustom):
    def encode(self, obj):
        return PyEncoder().dumpb(obj)


class BenchmarkJSONEncoder_Marshal(BaseBenchmarkJSONEncoder):
    def benchmark_array_256_decimals(self):
        skip()

    def benchmark_array_256_deriveddict_string_int(self):
        skip()

    def benchmark_array_256_ordereddict_string_int(self):
        skip()

    def encode(self, obj):
        return marshal.dumps(obj)



##
# Copyright (c) 2012 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


import sys
from setuptools import setup, Extension


if sys.version_info[:2] < (3, 3):
    raise RuntimeError('metamagic.json requires python 3.3 or greater')


readme = open('README.rst').read()


setup(
    name='metamagic.json',
    version='0.9.3',
    description='Fast JSON encoder',
    long_description=readme,
    maintainer='Sprymix Inc.',
    maintainer_email='info@sprymix.com',
    license='BSD',
    url='http://github.com/sprymix/metamagic.json',
    platforms=['any'],
    keywords='json',
    ext_modules=[
        Extension('metamagic.json._encoder',
                  sources=['metamagic/json/_encoder/_encoder.c'],
                  extra_compile_args=['-O3'])
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: C',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3'
    ],
    packages=[
        'metamagic.json',
        'metamagic.json._encoder',
        'metamagic.json.tests'
    ],
    include_package_data=True
)



##
# Copyright (c) 2012 Sprymix Inc.
# All rights reserved.
#
# See LICENSE for details.
##


try:
    from metamagic.test import skipif
except ImportError:
    from pytest import mark
    def skipif(cond):
        assert cond in (True, False)
        return mark.skipif(repr(cond))

from json import loads as std_loads, dumps as std_dumps
from decimal import Decimal
from collections import OrderedDict, Set, Sequence, Mapping
from uuid import UUID
import random
from datetime import datetime, tzinfo, timedelta, date, time

from metamagic.utils.debug import assert_raises


class _BaseJsonEncoderTest:
    encoder = None

    def __new__(cls):
        obj = super().__new__(cls)
        obj.dumps = lambda *args, **kwargs: cls.encoder().dumps(*args, **kwargs)
        obj.dumpb = lambda *args, **kwargs: cls.encoder().dumpb(*args, **kwargs)
        return obj

    def encoder_test(self, obj, expected, convertable_back = True, matches_default_encoder = True):
        """Tests that our dumps behaves exactly like the default json module
        Should not be used for cases where we don't like the default module behavior
        """
        if expected is None:
            expected = expected_b = {None}
        else:
            if isinstance(expected, str):
                expected = {expected}
            else:
                expected = set(expected)

        encoded = self.dumps(obj)
        assert encoded in expected

        expected_b = {e.encode('ascii') for e in expected}
        encoded_b = self.dumpb(obj)
        assert encoded_b in expected_b

        if convertable_back:
            # test decode: not always possible, e.g. for Decimals or integers larger than Java-max
            assert std_loads(self.dumps(obj)) == obj

        if matches_default_encoder:
            # test compliance with standard encoder; use compact encoding (set
            #  separators) - ow lists are expected with extra spaces between items
            assert std_dumps(obj, separators=(',',':')) == self.dumps(obj)

    def test_json_encoder_literals(self):
        self.encoder_test(None,  'null')

        self.encoder_test(True,  'true')
        self.encoder_test(False, 'false')

        self.encoder_test('foo', '"foo"')
        self.encoder_test('f"o"o', '"f\\"o\\"o"')
        self.encoder_test('foo\nbar', '"foo\\nbar"')
        self.encoder_test('\tf\\oo\n ', '"\\tf\\\\oo\\n "')

        self.encoder_test('\x00', '"\\u0000"')
        self.encoder_test('a\x19b', '"a\\u0019b"')
        self.encoder_test('a\x20b', '"a b"')

        self.encoder_test('\x00a\nbcعمان', '"\\u0000a\\nbc\\u0639\\u0645\\u0627\\u0646"')

        # std json does not escape '/'
        self.encoder_test('/\\"\n\r\t\b\f', '"\/\\\\\\"\\n\\r\\t\\b\\f"', True, False)

        # HTML special chars are encoded to Unicode sequences by us, but not std json
        self.encoder_test('<a href="test?a&b">', r'"\u003ca href=\"test?a\u0026b\"\u003e"',
                          matches_default_encoder=False)

        # once a bug was introduced in the c version which passed all other tests but not
        # this one (for characted \u0638) - so need to keep this long UTF string as a
        # "more complicated" utf sample
        long_utf_string = "عالم " * 50
        assert std_dumps(long_utf_string, separators=(',',':')) == self.dumps(long_utf_string)

    def test_json_encoder_numbers(self):
        self.encoder_test(0, '0')
        self.encoder_test(1, '1')
        self.encoder_test(-123456, '-123456')

        self.encoder_test(2.2, '2.2')
        self.encoder_test(1.2345678, '1.2345678')
        self.encoder_test(-1.23456e-123, '-1.23456e-123')

        # 9007199254740992 is the largest possible integer in JavaScript
        self.encoder_test(9007199254740992, '9007199254740992')

        # std module just returns a string representation for
        # all valid python numbers (even for non-jscript representable numbers)
        with assert_raises(ValueError, error_re='out of range'):
            self.encoder_test( 9007199254740992 + 1, None)
        with assert_raises(ValueError, error_re='out of range'):
            self.encoder_test(-9007199254740992 - 1, None)
        with assert_raises(ValueError, error_re='NaN is not supported'):
            self.encoder_test(float('NaN'), None)

        # std module does not support Decimals
        # note: Decimal(1.17) != Decimal("1.17"), for testing need initialization from a string
        self.encoder_test(Decimal("1.17"), '"1.17"', False, False)

        lst = [Decimal(str(random.random()*100000)) for _ in range(256)]
        out = '[{}]'.format(','.join('"{}"'.format(str(d)) for d in lst))
        self.encoder_test(lst, out, False, False)

        # complex numbers are not JSON serializable
        with assert_raises(TypeError, error_re='not JSON serializable'):
            self.encoder_test(1+2j, None)

    def test_json_encoder_lists(self):
        self.encoder_test([],                '[]')
        self.encoder_test(['abc',True],      '["abc",true]')
        self.encoder_test(['abc',[],[True]], '["abc",[],[true]]')

        # a tuple is converted to the same string as a "similar" list, so no conversion back
        # (but should match the default encoder)
        self.encoder_test((), '[]',                               False, True)
        self.encoder_test((1,), '[1]',                            False, True)
        self.encoder_test((1,2,[1,2,3]), '[1,2,[1,2,3]]',         False, True)
        self.encoder_test(('a','b',('c',1)), '["a","b",["c",1]]', False, True)

        self.encoder_test((99,100,999,1000,9999,10000,99999,100000),
                          '[99,100,999,1000,9999,10000,99999,100000]', False, True)

        # std module does not support sets
        self.encoder_test({1,2,3}, '[1,2,3]', False, False)

        # test max recursion level checks
        with assert_raises(ValueError, error_re='Exceeded maximum allowed recursion level'):
            self.dumps([[[1,2],3],4], max_nested_level=1)
        with assert_raises(ValueError, error_re='Exceeded maximum allowed recursion level'):
            self.dumps([[[1,2],3],4], max_nested_level=2)
        # this should work
        assert(self.dumps([[[1,2],3],4], max_nested_level=3) == '[[[1,2],3],4]')
        # create infinite recursion
        a=[1]
        a[0]=a
        with assert_raises(ValueError, error_re='Exceeded maximum allowed recursion level'):
            self.dumps(a)

        # by design type "bytes" is not serializable and should raise a TypeError
        with assert_raises(TypeError, error_re='not JSON serializable'):
            self.encoder_test(bytes([1,2,3]), None)

        # by design type "bytearray" is not serializable and should raise a TypeError
        with assert_raises(TypeError, error_re='not JSON serializable'):
            self.encoder_test(bytearray([1,2,3]), '[1,2,3]', False, False)

    def test_json_encoder_dict(self):
        self.encoder_test({}, '{}')
        self.encoder_test({'foo':1, 'bar':2}, ('{"foo":1,"bar":2}', '{"bar":2,"foo":1}'))
        self.encoder_test({'foo':[1,2], 'bar':[3,4]}, ('{"foo":[1,2],"bar":[3,4]}',
                                                       '{"bar":[3,4],"foo":[1,2]}'))

        # shold match std encoder, but conversion back converts to lists not tuples
        self.encoder_test({'foo':(1,2), 'bar':(3,4)}, ('{"foo":[1,2],"bar":[3,4]}',
                                                       '{"bar":[3,4],"foo":[1,2]}'), False, True)

        # std encoder does not support sets
        self.encoder_test({'foo':{1,2}, 'bar':{3,4}}, ('{"foo":[1,2],"bar":[3,4]}',
                                                       '{"bar":[3,4],"foo":[1,2]}'), False, False)

        # std encoder does nto support OrderedDicts
        d = {'banana': 3, 'apple':4, 'pear': 1, 'orange': 2}
        ordered_d = OrderedDict(sorted(d.items(), key=lambda t: t[0]))
        self.encoder_test( ordered_d, '{"apple":4,"banana":3,"orange":2,"pear":1}', False, False)

        # JSON spec does not support keys which are not a string or a number
        with assert_raises(TypeError, error_re='is not a valid dictionary key'):
            self.encoder_test({1:1}, '{1:1}')
        with assert_raises(TypeError, error_re='is not a valid dictionary key'):
            self.encoder_test({(1,2):'a'}, '{[1,2]:"a"}')

        class DerivedDict(dict):
            pass
        self.encoder_test(DerivedDict({'foo':1, 'bar':2}), ('{"foo":1,"bar":2}',
                                                            '{"bar":2,"foo":1}'))

    def test_json_encoder_uuid(self):
        # std encodencoderer does not support UUIDs
        self.encoder_test(UUID('{12345678-1234-5678-1234-567812345678}'),
                         '"12345678-1234-5678-1234-567812345678"',
                         False, False)

        class MyUUID(UUID):
            pass

        self.encoder_test({UUID('{12345678-1234-5678-1234-567812345678}') : 1},
                         '{"12345678-1234-5678-1234-567812345678":1}',
                         False, False)

        self.encoder_test({MyUUID('{12345678-1234-5678-1234-567812345678}') : 1},
                         '{"12345678-1234-5678-1234-567812345678":1}',
                         False, False)

    def test_json_encoder_datetime(self):
        dt1 = datetime(2012,2,1,12,20,22,100)
        dt2 = datetime(1990,12,11,1,1,1,0)
        dt3 = datetime(2222,1,6,22,59,0,99999)

        # std encodr does not support datetime
        self.encoder_test(dt1, '"2012-02-01T12:20:22.000100"', False, False)
        self.encoder_test(dt2, '"1990-12-11T01:01:01"',        False, False)
        self.encoder_test(dt3, '"2222-01-06T22:59:00.099999"', False, False)

        dt = date(2012, 6, 1)
        self.encoder_test(dt, '"2012-06-01"', False, False)

        tm = time(12, 13, 14, 15)
        self.encoder_test(tm, '"12:13:14.000015"', False, False)

        # Marquesas Islands Time, UTC-09:30
        class MART(tzinfo):
            def utcoffset(self, dt):
                return timedelta(days=-1, seconds=(14 * 60 + 30) * 60)

            def tzname(self, dt):
                return "UTC-09:30"

            def dst(self, dt):
                return timedelta(0)

        mart = MART()
        dt3 = datetime(2012, 2, 1, 12, 20, 22, 100, mart)
        self.encoder_test(dt3, '"2012-02-01T12:20:22.000100-09:30"', False, False)

        tm5 = time(12, 13, 14, 15, mart)
        self.encoder_test(tm5, '"12:13:14.000015-09:30"', False, False)

        # Afghanistan Time, UTC+04:30
        class AFT(tzinfo):
            def utcoffset(self, dt):
                return timedelta(days=0, seconds=(4 * 60 + 30) * 60)

            def tzname(self, dt):
                return "UTC+04:30"

            def dst(self, dt):
                return timedelta(0)

        aft = AFT()
        dt3 = datetime(2012, 2, 1, 12, 20, 22, 100, aft)
        self.encoder_test(dt3, '"2012-02-01T12:20:22.000100+04:30"', False, False)

        tm5 = time(12, 13, 14, 15, aft)
        self.encoder_test(tm5, '"12:13:14.000015+04:30"', False, False)


    def test_json_encoder_special_ch(self):
        self.encoder_test('\x1b', '"\\u001b"')

    def test_json_encoder_custom_methods(self):
        class FooJson:
            def __mm_json__(self):
                return '{"foo":{"baz":"©","spam":"\x1b"}}'

        # none of these are suported by std encoder
        ex_result = '{"foo":{"baz":"\\u00a9","spam":"\\u001b"}}'
        self.encoder_test(FooJson(), ex_result, False, False)
        self.encoder_test({'bar':FooJson()}, '{"bar":' + ex_result + '}', False, False)

        class FooJsonb:
            def __mm_json__(self):
                return b'"foo"'

        # none of these are suported by std encoder
        self.encoder_test(FooJsonb(), '"foo"', False, False)
        self.encoder_test({'bar':FooJsonb()}, '{"bar":"foo"}', False, False)

        class FooJsonPlus:
            def __mm_json__(self):
                raise NotImplementedError

            def __mm_serialize__(self):
                return 'foo'

        # none of these are suported by std encoder
        self.encoder_test(FooJsonPlus(), '"foo"', False, False)
        self.encoder_test({'bar':FooJsonPlus()}, '{"bar":"foo"}', False, False)

        class Foo:
            def __mm_serialize__(self):
                return 'foo'

        # none of these are suported by std encoder
        self.encoder_test(Foo(), '"foo"', False, False)
        self.encoder_test({Foo():'bar'}, '{"foo":"bar"}', False, False)
        self.encoder_test({'bar':Foo()}, '{"bar":"foo"}', False, False)

        class FooNotImpl(str):
            def __mm_serialize__(self):
                raise NotImplementedError

        # none of these are suported by std encoder
        self.encoder_test(FooNotImpl('foo'), '"foo"', False, False)
        self.encoder_test({FooNotImpl('foo'):'bar'}, '{"foo":"bar"}', False, False)
        self.encoder_test({'bar':FooNotImpl('foo')}, '{"bar":"foo"}', False, False)

        class Spam:
            def __mm_serialize__(self):
                1/0

        with assert_raises(ZeroDivisionError):
            self.dumps({'1': Spam()})

        with assert_raises(ZeroDivisionError):
            self.dumps({Spam(): 1})

    def test_json_default(self):
        class Foo:
            pass

        class Bar:
            pass

        class Spam:
            pass

        class Encoder(self.encoder):
            def default(self, obj):
                if isinstance(obj, Foo):
                    return ['Foo', 'Foo']
                if isinstance(obj, Spam):
                    1/0
                return super().default(obj)

        assert Encoder().dumps([Foo(), Foo()]) == '[["Foo","Foo"],["Foo","Foo"]]'

        with assert_raises(TypeError, error_re='is not JSON seri'):
            Encoder().dumps(Bar())

        # let's check that the exceptions propagate fine
        with assert_raises(ZeroDivisionError):
            Encoder().dumps([Spam()])
        with assert_raises(ZeroDivisionError):
            Encoder().dumpb({Spam(): 1})

    def test_json_c_reftest(self):
        # test for leaked references
        big_list = []
        for _ in range(256):
            d = {str(random.random()*20): int(random.random()*1000000), str(random.random()*20): int(random.random()*1000000), str(random.random()*20): int(random.random()*1000000), str(random.random()*20): int(random.random()*1000000)}
            ordered_d = OrderedDict(sorted(d.items(), key=lambda t: t[0]))
            big_list.append(ordered_d)
        self.dumps(big_list)

        big_list2 = []
        for _ in range(256):
            big_list2.append(str(random.random()*100000))
        self.dumps(big_list2)

    def test_json_integer_output(self):
        for _ in range(5000):
            x = random.randint(-9007199254740992, 9007199254740992)
            assert std_dumps(x) == self.dumps(x)

    def test_json_abc_sequence(self):
        class seq(Sequence):
            len = 10
            def __getitem__(self, idx):
                if idx < self.len:
                    return idx ** 2
                raise IndexError
            def __len__(self):
                return self.len

        assert self.dumps(seq()) == '[0,1,4,9,16,25,36,49,64,81]'

    def test_json_abc_mapping(self):
        class map(Mapping):
            data = OrderedDict((('a', 'b'), ('c', 'd'), ('e', 'f')))
            def __getitem__(self, key):
                return self.data[key]
            def __len__(self):
                return len(self.data)
            def __iter__(self):
                return iter(self.data)

        assert self.dumps(map()) == self.dumps(map.data)

    def test_json_abc_set(self):
        class set(Set):
            items = [100, 20, 42]
            def __contains__(self, el):
                return el in self.items
            def __iter__(self):
                return iter(self.items)
            def __len__(self):
                return len(self.items)

        assert self.dumps(set()) == self.dumps(set.items)

    def test_json_hook(self):
        class Spam:
            pass

        class Encoder(self.encoder):
            def encode_hook(self, obj):
                if isinstance(obj, list):
                    return {str(i): el for i, el in enumerate(obj)}
                if isinstance(obj, int):
                    return '*' + str(obj)
                if isinstance(obj, Spam):
                    1/0
                return obj

        assert Encoder().dumps([1.11]) == '{"0":1.11}'
        assert Encoder().dumps([1])    == '{"0":"*1"}'

        with assert_raises(ZeroDivisionError):
            Encoder().dumps([Spam()])


from ..encoder import Encoder as PyEncoder

class TestPyJsonEncoder(_BaseJsonEncoderTest):
    encoder = PyEncoder


SKIPC = False
CEncoder = PyEncoder
try:
    from .._encoder import Encoder as CEncoder
except ImportError:
    SKIPC = True

@skipif(SKIPC)
class TestCJsonEncoder(_BaseJsonEncoderTest):
    encoder = CEncoder


def test_json_dump():
    #test bindings
    from metamagic.json import dumps, dumpb

    assert dumps(True) == 'true'
    assert dumpb(True) == b'true'
